#include "BayometricProxy.h"

#include "soapUranusCoreEndpointProxy.h"
#include "UranusCoreEndpoint.nsmap"

using namespace std;

UranusCoreEndpointProxy client;

BFSAuthResponse CreateSession(AuthRequestInfo authRequestInfo)
{
	ns1__AuthRequestInfo* authParam = new ns1__AuthRequestInfo();

	bool ShowGUI = authRequestInfo.ShowGUI == 0 ? false : true;
	int Timeout = authRequestInfo.Timeout;

	authParam->ShowGUI = &ShowGUI;
	authParam->Timeout = &Timeout;

	_ns1__CreateSessionResponse csResponse;
	_ns1__CreateSession* createSessionParams = new _ns1__CreateSession();
	createSessionParams->authRequestInfo = authParam;

	BFSAuthResponse response;

	int ret = client.CreateSession(createSessionParams, csResponse);

	if (ret == SOAP_OK)
	{
		response.Base.ResponseCode = *(csResponse.CreateSessionResult->ResponseCode);
		response.Base.ReturnMessage = *(csResponse.CreateSessionResult->ReturnMessage);
		if (response.Base.ResponseCode == ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SUCCESS)
		{
			response.SessionKey = *(csResponse.CreateSessionResult->SessionKey);
			response.PersonID = *(csResponse.CreateSessionResult->PersonID);
		}
	}
	else
	{
		response.Base.ResponseCode = ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SERVER_USCOREERROR;
	}

	return response;
}

BFSResponseBase EndSession(string sessionKey)
{
	_ns1__EndSessionResponse esResponse;
	_ns1__EndSession* endSessionParams = new _ns1__EndSession();
	endSessionParams->sessionKey = sessionKey != "" ? &sessionKey : NULL;

	BFSResponseBase response;

	int ret = client.EndSession(endSessionParams, esResponse);

	if (ret == SOAP_OK)
	{
		response.ResponseCode = *(esResponse.EndSessionResult->ResponseCode);
		response.ReturnMessage = *(esResponse.EndSessionResult->ReturnMessage);
	}
	else
	{
		response.ResponseCode = ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SERVER_USCOREERROR;
	}

	return response;
}

BFSResponse RegisterPerson(string sessionKey, string personID)
{
	_ns1__RegisterPersonResponse rpResponse;
	_ns1__RegisterPerson* registerPersonParams = new _ns1__RegisterPerson();
	registerPersonParams->sessionKey = sessionKey != "" ? &sessionKey : NULL;
	registerPersonParams->personID = personID != "" ? &personID : NULL;

	BFSResponse response;

	int ret = client.RegisterPerson(registerPersonParams, rpResponse);

	if (ret == SOAP_OK)
	{
		response.Base.ResponseCode = *(rpResponse.RegisterPersonResult->ResponseCode);
		response.Base.ReturnMessage = *(rpResponse.RegisterPersonResult->ReturnMessage);
		if (response.Base.ResponseCode == ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SUCCESS)
		{
			response.PersonID = rpResponse.RegisterPersonResult->PersonID != NULL ? *(rpResponse.RegisterPersonResult->PersonID) : "";
			response.PersonFoundID = rpResponse.RegisterPersonResult->PersonFoundID != NULL ? *(rpResponse.RegisterPersonResult->PersonFoundID) : "";
		}
	}
	else
	{
		response.Base.ResponseCode = ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SERVER_USCOREERROR;
	}

	return response;
}

BFSResponse UnregisterPerson(string sessionKey, string personID)
{

	_ns1__UnregisterPersonResponse upResponse;
	_ns1__UnregisterPerson* unregisterPersonParams = new _ns1__UnregisterPerson();
	unregisterPersonParams->sessionKey = sessionKey != "" ? &sessionKey : NULL;
	unregisterPersonParams->personID = personID != "" ? &personID : NULL;

	BFSResponse response;

	int ret = client.UnregisterPerson(unregisterPersonParams, upResponse);

	if (ret == SOAP_OK)
	{
		response.Base.ResponseCode = *(upResponse.UnregisterPersonResult->ResponseCode);
		response.Base.ReturnMessage = *(upResponse.UnregisterPersonResult->ReturnMessage);

	}
	else
	{
		response.Base.ResponseCode = ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SERVER_USCOREERROR;
	}

	return response;
}

BFSResponse Search()
{
	/*Since search may run on a different thread,
	gsoap says we must use a different client context.*/
	UranusCoreEndpointProxy searchClient;

	_ns1__SearchResponse sResponse;
	_ns1__Search* searchParams = new _ns1__Search();

	BFSResponse response;

	int ret = searchClient.Search(searchParams, sResponse);

	if (ret == SOAP_OK)
	{
		response.Base.ResponseCode = *(sResponse.SearchResult->ResponseCode);
		response.Base.ReturnMessage = *(sResponse.SearchResult->ReturnMessage);
		if (response.Base.ResponseCode == ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__HIT_USCORECONFIRMED)
		{
			response.PersonFoundID = sResponse.SearchResult->PersonFoundID != NULL ?  *(sResponse.SearchResult->PersonFoundID) : "";
		}
	}
	else
	{
		response.Base.ResponseCode = ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SERVER_USCOREERROR;
	}

	return response;
}

BFSResponse SearchEx(string appToken, string personID, int showGUI, int timeout)
{
	/*Since search may run on a different thread, gsoap says we must use a different client context.*/
	UranusCoreEndpointProxy searchClient;

	_ns1__SearchExResponse sResponse;
	_ns1__SearchEx* searchParams = new _ns1__SearchEx();

	bool ShowGUI = showGUI == 0 ? false : true;
	int Timeout = timeout;

	searchParams->appToken = appToken != "" ? &appToken : NULL;

	searchParams->showGUI = &ShowGUI;
	searchParams->timeout = &timeout;

	BFSResponse response;

	int ret = searchClient.SearchEx(searchParams, sResponse);

	if (ret == SOAP_OK)
	{
		response.Base.ResponseCode = *(sResponse.SearchExResult->ResponseCode);
		response.Base.ReturnMessage = *(sResponse.SearchExResult->ReturnMessage);
		if (response.Base.ResponseCode == ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__HIT_USCORECONFIRMED)
		{
			response.PersonFoundID = sResponse.SearchExResult->PersonFoundID != NULL ? *(sResponse.SearchExResult->PersonFoundID) : "";
		}
	}
	else
	{
		response.Base.ResponseCode = ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SERVER_USCOREERROR;
	}

	return response;
}

BFSResponse CancelSearch(string appToken)
{

	_ns1__CancelSearchResponse csResponse;
	_ns1__CancelSearch* cancelSearchParams = new _ns1__CancelSearch();

	cancelSearchParams->appToken = appToken != "" ? &appToken : NULL;

	BFSResponse response;

	int ret = client.CancelSearch(cancelSearchParams, csResponse);

	if (ret == SOAP_OK)
	{
		response.Base.ResponseCode = *(csResponse.CancelSearchResult->ResponseCode);
		response.Base.ReturnMessage = *(csResponse.CancelSearchResult->ReturnMessage);
	}
	else
	{
		response.Base.ResponseCode = ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SERVER_USCOREERROR;
	}

	return response;
}

BFSInfoResponse GetRegisteredPersons(int startIndex, int numberOfPersons)
{
	_ns1__GetRegisteredPersonsResponse grpResponse;
	_ns1__GetRegisteredPersons* getRegisteredPersonsParam = new _ns1__GetRegisteredPersons();
	
	int StartIndex = startIndex;
	int NumberOfPersons = numberOfPersons;
	
	getRegisteredPersonsParam->startIndex = &StartIndex;
	getRegisteredPersonsParam->numberOfPersons = &NumberOfPersons;

	BFSInfoResponse response;

	int ret = client.GetRegisteredPersons(getRegisteredPersonsParam, grpResponse);

	if (ret == SOAP_OK)
	{
		response.Base.ResponseCode = *(grpResponse.GetRegisteredPersonsResult->ResponseCode);
		response.Base.ReturnMessage = *(grpResponse.GetRegisteredPersonsResult->ReturnMessage);

		if (response.Base.ResponseCode == ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SUCCESS || response.Base.ResponseCode == ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__END_USCOREOF_USCORELIST)
		{
			
			for (int i = 0; i < grpResponse.GetRegisteredPersonsResult->RegisteredPersons->string.size(); i++)
			{
				response.RegisteredPersons.push_back(grpResponse.GetRegisteredPersonsResult->RegisteredPersons->string[i]);
			}
			
		}
	}
	else
	{
		response.Base.ResponseCode = ns1__BFSClientReturnErrorCode::ns1__BFSClientReturnErrorCode__SERVER_USCOREERROR;
	}

	
	return response;
}